#include <multiboot.h>
#include <video.h>
#include <memory.h>
#include <assert.h>
#include <system.h>
#include <stdlib.h>
#include <asm.h>
#include <stdio.h>
#include <keyboard.h>
#include <console.h>
#include <labwars.h>
#include <timer.h>



/* 
 Remaps IRQ's in order to prevent collusion with 
 Intel's first 32 preserved CPU interrupts.
 
 irq0--> 21h , irq8--> 28h
   
 This code is taken from  Bran's Kernel Development Tutorial
 */
extern void remap_pics()
{
	outportb(0x20, 0x11);
    outportb(0xA0, 0x11);
    outportb(0x21, 0x20);
    outportb(0xA1, 0x28);
    outportb(0x21, 0x04);
    outportb(0xA1, 0x02);
    outportb(0x21, 0x01);
    outportb(0xA1, 0x01);
    outportb(0x21, 0x0);
    outportb(0xA1, 0x0);
}

//overload the operator "new"
/*void * operator new (size_t size)
{
	return kalloc(size);
}
*/

//Program *program = new Program(&Memory::Modules[0]);
/*class Program
{
	private:

	Module *module;
	

	public:

	Program(Module *module)
	{
		assert(module != NULL);
		this->module = module;
	}	
};*/



// Handle Timer
extern void timer_handler()
{
	if(LabWars::isGameInited() == 1)
		LabWars::run_game_engine();
	
	Timer::inc_tick();
}


// Handle Keyboard
extern void keyboard_handler()
{
	unsigned char scancode = inportb(0x60); // Get keyboard code
	
	// Handle keyboard scan codes
	Keyboard::handle_keyboard(scancode);
	
	// If game is not running....Run Console
	if(LabWars::isGameInited() == 0)	
		Console::runConsole(scancode);
	
	//If game is initialized, start to get game keys...
	if(LabWars::isGameInited() == 1)
		LabWars::handle_keys(scancode);
}


// Main
int main(multiboot_info *boot_info)
{
	Video::Clear();
	Memory::Initialize(boot_info);
	printk("Memory initialized....\n");
	Keyboard::init();
	printk("Keyboard initialized...\n");
	printk("Starting OS console...\n");
	Console::initConsole();
	
	//Program *program = new Program(&Memory::Modules[0])

	return 0;
}
