#include <keyboard.h>
#include <system.h>
#include <ctype.h>

int Keyboard::ctrl;
int Keyboard::shift;
int Keyboard::alt;
int Keyboard::capsLock;
int Keyboard::asciiChar;
int Keyboard::tempIndex; 

char Keyboard::keyboardBuffer[BUFF_SIZE];

// Part of this keyboard map is taken from 
// Tutorial in page : http://www.osdever.net/bkerndev/index.php?the_id=90
// This is a plain keyboard map withour shift key
char Keyboard::keyboardMap[] = 
{
    0,  27, '1', '2', '3', '4', '5', '6', '7', '8',	/* 9 */
  '9', '0', '-', '=', '\b',	/* Backspace */
  '\t',			/* Tab */
  'q', 'w', 'e', 'r',	/* 19 */
  't', 'y', 'u', 'i', 'o', 'p', '[', ']', '\n',	/* Enter key */
    0,			/* 29   - Control */
  'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',	/* 39 */
 '\'', '`',   0,		/* Left shift */
 '\\', 'z', 'x', 'c', 'v', 'b', 'n',			/* 49 */
  'm', ',', '.', '/',   0,				/* Right shift */
  '*',
    0,	/* Alt */
  ' ',	/* Space bar */
    0,	/* Caps lock */
    0,	/* 59 - F1 key ... > */
    0,   0,   0,   0,   0,   0,   0,   0,
    0,	/* < ... F10 */
    0,	/* 69 - Num lock*/
    0,	/* Scroll Lock */
    0,	/* Home key */
    0,	/* Up Arrow */
    0,	/* Page Up */
  '-',
    0,	/* Left Arrow */
    0,
    0,	/* Right Arrow */
  '+',
    0,	/* 79 - End key*/
    0,	/* Down Arrow */
    0,	/* Page Down */
    0,	/* Insert Key */
    0,	/* Delete Key */
    0,   0,   0,
    0,	/* F11 Key */
    0,	/* F12 Key */
    0,	/* All other keys are undefined */
};

// Keyboard map with shift key
char Keyboard::keyboardMapWithShift[] = 
{
    0,  27, '!', '2', '3', '4', '%', '^', '&', '8',	/* 9 */
  '(', ')', '_', '+', '\b',	/* Backspace */
  '\t',			/* Tab */
  'q', 'w', 'e', 'r',	/* 19 */
  't', 'y', 'u', 'i', 'o', 'p', '[', ']', '\n',	/* Enter key */
    0,			/* 29   - Control */
  'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',	/* 39 */
 '\'', '`',   0,		/* Left shift */
 '\\', 'z', 'x', 'c', 'v', 'b', 'n',			/* 49 */
  'm', ',', '.', '?',   0,				/* Right shift */
  '*',
    0,	/* Alt */
  ' ',	/* Space bar */
    0,	/* Caps lock */
    0,	/* 59 - F1 key ... > */
    0,   0,   0,   0,   0,   0,   0,   0,
    0,	/* < ... F10 */
    0,	/* 69 - Num lock*/
    0,	/* Scroll Lock */
    0,	/* Home key */
    0,	/* Up Arrow */
    0,	/* Page Up */
  '-',
    0,	/* Left Arrow */
    0,
    0,	/* Right Arrow */
  '+',
    0,	/* 79 - End key*/
    0,	/* Down Arrow */
    0,	/* Page Down */
    0,	/* Insert Key */
    0,	/* Delete Key */
    0,   0,   0,
    0,	/* F11 Key */
    0,	/* F12 Key */
    0,	/* All other keys are undefined */
};


void Keyboard::init()
{
	ctrl	= 0;
	shift	= 0;
	alt   	= 0;
	capsLock  = 0;
	asciiChar = 0;
	tempIndex = 0;
	
	for(int i = 0; i < BUFF_SIZE; ++i)
		keyboardBuffer[i] = '\0'; // Fill with null chars
}

void Keyboard::handle_keyboard(unsigned char scancode)
{
	if(scancode & 0x80) // If a key released
    {
    	// what is released?
		if(scancode == 0x9d) // CTRL
			ctrl = 0;
		
		else if(scancode == 0xb8) // ALT
			alt = 0;
		
		else if(scancode == 0xb6 || scancode == 0xaa) // SHIFT
			shift = 0;
		
		asciiChar = 0; // Clear ascii char
    }
	
    else // a key is pressed 
    {
		// Handle special flag based keys ?
		if(scancode == 0x1d) // CTRL
			ctrl = 1;
		
		else if(scancode == 0x38) // ALT
			alt = 1;
		
		else if(scancode == 0x36 || scancode == 0x2a) // SHIFT
			shift = 1;
		
		else if(scancode == 0x3a) // CAPSLOCK
		{
			if(capsLock == 1) 
				capsLock = 0;
			else
				capsLock = 1;
		}
		
		if(ctrl == 1 && alt == 1 && scancode == 0x53)
			System::Reboot();
		
		// Look up ASCII char tables 
		asciiChar = handle_keyboard_map(scancode);
	
		// Eleminate whitespace chars
		if(asciiChar != 0 && asciiChar != '\n' && asciiChar != '\t'
			&& asciiChar != 27)
			handle_keyboard_buffer(asciiChar);
	}
}

char Keyboard::handle_keyboard_map(unsigned char scancode)
{
	char ch = 0;
	
	if(scancode < 128)
	{
		if(shift == 1)
			ch = keyboardMapWithShift[scancode];
		
		else
			ch = keyboardMap[scancode];
		
		// Shift print opposite of Capslock, Upper x Lower 
		if(shift == 0 && capsLock == 1) ch = toupper(ch);
		if(shift == 1 && capsLock == 0) ch = toupper(ch);

	}
	
	return ch;
}

// Handle keyboard buffer
void Keyboard::handle_keyboard_buffer(char ch)
{		
	if(asciiChar == '\b')
	{
		keyboardBuffer[tempIndex] = '\0'; // Add null
		tempIndex--;
	}
	
	else
	{
		keyboardBuffer[tempIndex] = asciiChar;
		tempIndex ++;
	}
			
		// Range controls
	if(tempIndex <= 0) tempIndex = 0;
	if(tempIndex >= BUFF_SIZE) tempIndex = BUFF_SIZE - 1;
}

//Get keyboard buffer
char* Keyboard::get_keyboard_buffer()
{
	return keyboardBuffer;
}


// Return ascii char 
char Keyboard::get_ascii_char()
{
	return asciiChar;
}

// After command is executed, we clear keyboard buffer
void Keyboard::clear_keyboard_buffer()
{
	for(int i=0; i < BUFF_SIZE; ++i)
		keyboardBuffer[i] = '\0'; // fill with null;
	
	tempIndex = 0;
}

// Return flag keys , ALT, SHIFT, CTRL, CAPSLOCK
int Keyboard::get_alt_key()
{
	return alt;
}

int Keyboard::get_shift_key()
{
	return shift;
}

int Keyboard::get_ctrl_key()
{
	return ctrl;
}

int Keyboard::get_capslock_key()
{
	return capsLock;
}
